package com.gic.demo.single;

import com.gic.demo.single.service.DemoStoreService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.function.Consumer;

/**
 * @author zhurz
 */
@Component
public class TestRun {

    @Value("${jdbc.maxsize}")
    private Integer jdbcMaxsize;

    public void test1(Consumer<String> consumer, int maxThreadSize, int size) throws InterruptedException {
//        int maxThreadSize = 100;
//        int size = 100000;
        ExecutorService exec = Executors.newFixedThreadPool(maxThreadSize);
        CountDownLatch countDownLatch = new CountDownLatch(size);
        long begin = System.currentTimeMillis();
        for (int i = 0; i < size; i++) {
            int finalI = i;
            exec.execute(() -> {
                try {
                    consumer.accept("sss" + finalI);
                } finally {
                    countDownLatch.countDown();
                }
            });
        }
        countDownLatch.await();
        long now = System.currentTimeMillis();
        double cost = ((now - begin) / 1000d);
        System.out.printf(
                "并发线程: %d, jdbc最大连接: %d, 查询次数: %d, 耗时:%.2f s, QPS: %.2f\n",
                maxThreadSize,
                jdbcMaxsize,
                size,
                cost,
                size / cost
        );
        exec.shutdown();
    }

    public void test2(DemoStoreService demoStoreService) {
        System.out.println("无事务");
        demoStoreService.test1("123");
        System.out.println("开启事务");
        demoStoreService.test2("456");
        System.out.println("只读事务");
        demoStoreService.test3("789");
    }

    public void test3(DemoStoreService demoStoreService, int maxThreadSize, int size, int choose) throws InterruptedException {
        boolean all = choose == 0;
        int batchSizeMax = 100;
        int step = size / batchSizeMax + (size % batchSizeMax > 0 ? 1 : 0);
        for (int i = 1; i <= step; i++) {
            int batchSize = i == step ? (size % batchSizeMax) : batchSizeMax;
            switch (choose) {
                case 0:
                case 1:
                    System.out.println("无事务");
                    test1(demoStoreService::test1, maxThreadSize, batchSize);
                    if (!all) break;
                case 2:
                    System.out.println("开启事务");
                    test1(demoStoreService::test2, maxThreadSize, batchSize);
                    if (!all) break;
                case 3:
                    System.out.println("只读事务");
                    test1(demoStoreService::test3, maxThreadSize, batchSize);
                    break;
            }
            Thread.sleep(25);
        }
    }

    public static void main(String[] args) throws InterruptedException, IOException {
        ApplicationContext applicationContext = new ClassPathXmlApplicationContext("applicationContext-conf.xml");
        TestRun bean = applicationContext.getBean(TestRun.class);
        DemoStoreService demoStoreService = applicationContext.getBean(DemoStoreService.class);
        BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
//        String s = reader.readLine();
//        System.out.println(s);
//        System.out.println("无事务");
//        bean.test1(demoStoreService::test1);
//        System.out.println("开启事务");
//        bean.test1(demoStoreService::test2);
//        System.out.println("只读事务");
//        bean.test1(demoStoreService::test3);
        int maxThreadSize = 100;
        int size = 10000;
        int choose = 0;
        String input;
        while (true) {
            System.out.printf("请输入并发线程数，当前 %d\n", maxThreadSize);
            input = reader.readLine();
            if (input.length() > 0) {
                maxThreadSize = Integer.valueOf(input);
            }
            System.out.printf("请输入查询次数，当前 %d\n", size);
            input = reader.readLine();
            if (input.length() > 0) {
                size = Integer.valueOf(input);
            }
            System.out.printf("请输入测试项（当前 %d）\n" +
                    "0. 全部\n" +
                    "1. 无事务\n" +
                    "2. 开启事务\n" +
                    "3. 只读事务\n", choose);
            input = reader.readLine();
            if (input.length() > 0) {
                choose = Integer.valueOf(input);
            }
            System.out.println("-- test begin --");
            bean.test3(demoStoreService, maxThreadSize, size, choose);
            System.out.println("-- test end --");
            System.out.println("输入任意字符继续，exit退出...");
            input = reader.readLine();
            if ("exit".equals(input)) {
                System.exit(0);
                break;
            }
        }
    }

}
