package com.gic;

import com.alibaba.fastjson.JSON;
import com.dexcoder.commons.utils.UUIDUtils;
import com.gic.demo.single.dao.mapper.DemoEnterpriseMapper;
import com.gic.demo.single.dao.mapper.DemoStoreMapper;
import com.gic.demo.single.entity.TabGicDemoEnterprise;
import com.gic.demo.single.entity.TabGicDemoStore;
import com.gic.demo.single.service.DemoStoreService;
import org.apache.commons.lang3.RandomStringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Unit test for simple App.
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = {"classpath*:applicationContext-conf.xml"})
public class AppTest {

    @Autowired
    private DemoEnterpriseMapper enterpriseMapper;

    @Autowired
    private DemoStoreMapper storeMapper;

    @Autowired
    private DemoStoreService demoStoreService;

    @Test
    public void test1() {
        TabGicDemoEnterprise tabGicDemoEnterprise = enterpriseMapper.selectByPrimaryKey("22323423");
        System.out.println(JSON.toJSONString(tabGicDemoEnterprise, true));
    }

    @Test
    public void test2() {
        TabGicDemoStore tabGicDemoStore = storeMapper.selectByPrimaryKey("11b");
        System.out.println(JSON.toJSONString(tabGicDemoStore, true));
    }

    @Test
    public void test3() {
        for (int i = 0; i < 10; i++) {
            System.out.println("insert >> " + i);
            TabGicDemoStore store = new TabGicDemoStore();
            if (i % 2 == 0) {
                store.setStoreId(UUIDUtils.getUUID32());
            }
            store.setStoreName(RandomStringUtils.random(10, "abcdefghigklmn0123456789"));
            storeMapper.insertSelective(store);
            System.out.println(JSON.toJSONString(store));
        }
    }

    @Test
    public void test4() {
        TabGicDemoStore store = demoStoreService.findDemoStoreById("001d854981019000");
        System.out.println(JSON.toJSONString(store, true));
    }


    @Value("${jdbc.maxsize}")
    private Integer jdbcMaxsize;

    @Test
    public void test5() throws InterruptedException {
        int maxThreadSize = 100;
        int size = 50000;
        ExecutorService exec = Executors.newFixedThreadPool(maxThreadSize);
        CountDownLatch countDownLatch = new CountDownLatch(size);
        long begin = System.currentTimeMillis();
        for (int i = 0; i < size; i++) {
            int finalI = i;
            exec.execute(() -> {
                try {
                    demoStoreService.findDemoStoreById("sss" + finalI);
                } finally {
                    countDownLatch.countDown();
                }
            });
        }
        countDownLatch.await();
        long now = System.currentTimeMillis();
        double cost = ((now - begin) / 1000d);
        System.out.printf(
                "并发线程: %d, jdbc最大连接: %d, 查询次数: %d, 耗时:%.2f s, QPS: %.2f",
                maxThreadSize,
                jdbcMaxsize,
                size,
                cost,
                size / cost
        );
    }

}
